package ch.rakudave.jnetmap.view.components;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;

import javax.swing.Action;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JToolBar;
import javax.swing.SwingUtilities;

import ch.rakudave.jnetmap.controller.Actions;
import ch.rakudave.jnetmap.util.Lang;
import ch.rakudave.jnetmap.util.Settings;
import ch.rakudave.jnetmap.util.logging.Logger;

/**
 * @author rakudave
 */
@SuppressWarnings("serial")
public class ToolBar extends JToolBar {
	public static final String defaultLayout = "new,open,save,|,undo,redo,|,map.pick,map.edit,map.transform,|,refresh,map.zoomin,map.zoomout,-,hamburger";
	private static final String identifierRegex = "[a-zA-Z\\.]*";
	private static ToolBar instance;
	private static HashMap<String, Action> actions = new HashMap<String, Action>();

	private ToolBar() {
		super(Settings.getInt("toolbar.orientation", 0));
		setVisible(Settings.getBoolean("toolbar.visible", true));
			actions.put("new", Actions.newMap(Lang.get("menu.file.new")));
			actions.put("open", Actions.open(Lang.get("menu.file.open")));
			actions.put("save", Actions.save());
			actions.put("saveas", Actions.saveAs());
			actions.put("doc", Actions.viewDoc(Lang.get("menu.help.doc")));
			actions.put("undo", Actions.undo());
			actions.put("redo", Actions.redo());
			actions.put("refresh", Actions.refresh());
	}
	
	/**
	 * @return an instance of ToolBar
	 */
	public static ToolBar getInstance() {
		if (instance == null) {
			synchronized (actions) {
				if (instance == null) {
					instance = new ToolBar();
					instance.rebuildToolbarLayout();
				}
			}
		}
		return instance;
	}
	
	@Override
	public void setVisible(boolean aFlag) {
		super.setVisible(aFlag);
		Settings.put("toolbar.visible", aFlag);
	}
	
	@Override
	public void setOrientation(int o) {
		super.setOrientation(o);
		Settings.put("toolbar.orientation", o);
	}
	
	/**
	 * Add an action to the toolbar (as a button).
	 * It will only be displayed if the toolbar-settings say so.
	 * @param identifier Unique identifier, used in the toolbar-settings
	 * 		It may only consist of letters and dots, matching [a-zA-Z\\.]*
	 * @param action Button-action
	 * @return Success. If the identifier already exists or does not match 
	 * 		the expected format, it will not be added.
	 */
	public boolean add(String identifier, Action action) {
		if (identifier == null || action == null || 
				actions.containsKey(identifier) || 
				!identifier.matches(identifierRegex)) {
			return false;
		} else {
			actions.put(identifier, action);
			rebuildToolbarLayout();
			return true;
		}
	}
	
	public void remove(String identifier) {
		actions.remove(identifier);
	}
	
	@SuppressWarnings("unchecked")
	public HashMap<String, Action> getActions() {
		return (HashMap<String, Action>) actions.clone();
	}
	
	/**
	 * Rebuilds the toolbars layout based on the configuration-strings in toolbar.layout
	 */
	public synchronized void rebuildToolbarLayout() {
		instance.removeAll();
		List<String> layout = Arrays.asList(Settings.get("toolbar.layout", defaultLayout).split(","));
		for (String id : layout) {
			if ("|".equals(id)) {
				instance.addSeparator();
			} else if ("-".equals(id)) {
				instance.add(Box.createHorizontalGlue());
			} else if (id.matches(identifierRegex) && layout.contains(id) && actions.containsKey(id)) {
				Action a = actions.get(id);
				JButton b = new JButton(a);
				if (!Settings.getBoolean("toolbar.labels", false)) {
					b.setText("");
					b.setToolTipText((String) a.getValue(Action.NAME));
				}
				instance.add(b);
			} else {
				Logger.trace("Unknown toolbar action "+id);
			}
		}
		SwingUtilities.updateComponentTreeUI(this);
	}
}
